/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "board.h"

#include <config.h>

#include <kio/slaveconfig.h>
#include <kio/netaccess.h>
#include <kdebug.h>

#include <kfilterdev.h>

#include <qregexp.h>
#include <qdict.h>

#include "kita_misc.h"
#include "threadinfo.h"

#include "qcp932codec.h"

enum {
    Board_MachiBBS,
    Board_2ch
};

using namespace Kita;

QDict<Board>* Board::m_boardDict = 0;

Board::Board()
{}

Board::Board( const QString& boardURL, const QString boardName ) : m_boardURL( boardURL ), m_boardName( boardName )
{}

Board::~Board()
{}

const KURL Board::subjectTxtURL() const
{
    if ( m_boardURL.isEmpty() ) {
        return KURL();
    } else {
        return KURL( m_boardURL, "./subject.txt" );
    }
}

const QString& Board::url() const
{
    return m_boardURL;
}

const QString& Board::name() const
{
    return m_boardName;
}

const QString Board::id() const
{
    return KURL( m_boardURL ).fileName();
}

int Board::type() const
{
    QRegExp url_machibbs( "http://.*\\.machi\\.to/.*" );
    if ( url_machibbs.search( m_boardURL ) == -1 ) {
        return Board_2ch;
    } else {
        return Board_MachiBBS;
    }
}

const QPtrList<Thread> Board::getThreadList() const
{
    if ( m_boardURL.isEmpty() ) {
        kdWarning() << "Waring: board url is empty!" << endl;
        return QPtrList<Thread>();
    }
    QPtrList<Thread> threadList;

    KIO::SlaveConfig::self() ->setConfigData( "http",
            KURL( m_boardURL ).host() ,
            "UserAgent",
            QString( "Monazilla/1.00 (Kita/%1)" ).arg( VERSION ) );
    QString tmpFile;
    if ( KIO::NetAccess::download( subjectTxtURL(), tmpFile ) ) {
        QIODevice * tmpDevice = KFilterDev::deviceForFile( tmpFile, "application/x-gzip" );
        tmpDevice->open( IO_ReadOnly );

        QCp932Codec cp932Codec;
        QTextStream stream( tmpDevice );
        stream.setCodec( &cp932Codec );

        // parse subject.txt(only one format...)
        // FIXME: need to refactoring
        QRegExp regexp;
        switch ( type() ) {
        case Board_MachiBBS:
            regexp.setPattern( "(\\d+\\.cgi),(.*)\\((\\d+)\\)" );
            break;
        case Board_2ch:
        default:
            regexp.setPattern( "(\\d+\\.dat)<>(.*)\\((\\d+)\\)" );
            break;
        }
        QString line;

        while ( ( line = stream.readLine() ) != QString::null ) {
            int pos = regexp.search( line );
            if ( pos != -1 ) {
                QString fname = regexp.cap( 1 );
                QString subject = regexp.cap( 2 );
                QString num = regexp.cap( 3 );

                KURL datURL = m_boardURL;
                datURL.addPath( "/dat/" + fname );
                Kita::Board::setName( datToBoard( datURL.url() ), name() );
                Kita::Thread::setName( datURL.url(), subject );
                Kita::Thread* thread = Kita::Thread::getByURL( datURL.url() );
                KitaThreadInfo::setResNum( datURL.url(), num.toInt() );
                threadList.append( thread );
            }
        }

        KIO::NetAccess::removeTempFile( tmpFile );
    }
    return threadList;
}

Board* Board::getByURL( const QString& boardURL )
{
    // FIXME: null֥Ȥ׵ΤӤ˺äƤ롣
    if ( boardURL.isNull() ) {
        return new NullBoard();
    }

    if ( m_boardDict == 0 ) {
        // FIXME: setAutoDeleteäưȤΤ뤳
        m_boardDict = new QDict<Board>();
    }

    if ( m_boardDict->find( boardURL ) ) {
        return m_boardDict->find( boardURL );
    }

    Board* newBoard = new Board( boardURL );
    m_boardDict->insert( boardURL, newBoard );

    return newBoard;
}

void Board::setName( const QString& boardURL, const QString& boardName )
{
    if ( boardURL.isNull() ) {
        return ;
    }

    if ( m_boardDict == 0 ) {
        // FIXME: setAutoDeleteäưȤΤ뤳
        m_boardDict = new QDict<Board>();
    }

    if ( m_boardDict->find( boardURL ) ) {
        m_boardDict->find( boardURL ) ->m_boardName = boardName;
    } else {
        Board* newBoard = new Board( boardURL, boardName );
        m_boardDict->insert( boardURL, newBoard );
    }
    return ;
}

const QString& Board::getName( const QString& boardURL )
{
    return Board::getByURL( boardURL ) ->name();
}

NullBoard::NullBoard()
{}

NullBoard::~ NullBoard()
{}
