/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "favoritethreads.h"

#include <kdebug.h>

#include <qstylesheet.h>
#include <qdom.h>

#include "threadinfo.h"
#include "board.h"
#include "kita_misc.h"

FavoriteThreads* FavoriteThreads::instance = 0;

FavoriteThreads::FavoriteThreads()
{}

FavoriteThreads::~FavoriteThreads()
{}

FavoriteThreads* FavoriteThreads::getInstance()
{
    if ( ! instance ) {
        instance = new FavoriteThreads();
    }
    return instance;
}

void FavoriteThreads::insert( const Kita::Thread* thread )
{
    if ( ! m_threadDict.find( thread->datURL() ) ) {
        m_threadDict.insert( thread->datURL(), thread );
    }
}

void FavoriteThreads::remove( const QString& datURL )
{
    m_threadDict.remove( datURL );
}

bool FavoriteThreads::contains( const QString& datURL ) const
{
    if ( m_threadDict.find( datURL ) ) {
        return true;
    } else {
        return false;
    }
}

bool FavoriteThreads::readFromXML( const QString& xml )
{
    FavoriteThreads * instance = FavoriteThreads::getInstance();
    instance->m_threadDict.clear();

    QDomDocument document;
    if ( ! document.setContent( xml, true ) ) {
        return false;
    }

    QDomElement root = document.documentElement();

    QDomNode node = root.firstChild();
    while ( !node.isNull() ) {
        if ( node.isElement() &&
                node.nodeName() == QString( "thread" ) &&
                node.namespaceURI() == QString( "http://kita.sourceforge.jp/ns/thread" ) ) {
            processThreadNode( node );
        }
        node = node.nextSibling();
    }
    return true;
}

void FavoriteThreads::processThreadNode( QDomNode& node )
{
    QDomNode datURLNode = node.namedItem( "daturl" );
    QDomNode nameNode = node.namedItem( "name" );
    QDomNode resNumNode = node.namedItem( "resnum" );
    QDomNode boardNode = node.namedItem( "board" );

    if ( !datURLNode.isElement() || !nameNode.isElement() ||
            !resNumNode.isElement() || !boardNode.isElement() ) {
        return ;
    }
    if ( boardNode.namespaceURI() != QString( "http://kita.sourceforge.jp/ns/board" ) ) {
        return ;
    }
    if ( !processBoardNode( boardNode ) ) {
        return ;
    }
    QString datURLText = datURLNode.toElement().text();
    QString nameText = nameNode.toElement().text();
    int resNum = resNumNode.toElement().text().toInt();

    Kita::Thread::setName( datURLText, QStyleSheet::escape( nameText ) );
    KitaThreadInfo::setResNum( datURLText, resNum );

    FavoriteThreads::getInstance() ->insert( Kita::Thread::getByURL( datURLText ) );
}

bool FavoriteThreads::processBoardNode( QDomNode& node )
{
    QDomNode urlNode = node.namedItem( "url" );
    QDomNode nameNode = node.namedItem( "name" );
    if ( ! urlNode.isElement() || ! nameNode.isElement() ) {
        return false;
    }

    QString urlText = urlNode.toElement().text();
    QString nameText = nameNode.toElement().text();

    KURL url = KURL( urlText );
    if ( url.isValid() ) {
        Kita::Board::setName( urlText, nameText );
        return true;
    }
    return false;
}

const QString FavoriteThreads::toXML() const
{
    QDomDocument document;
    
//    QDomProcessingInstruction pi = document.createProcessingInstruction( "xml", "version=\"1.0\"" );
//    document.appendChild( pi );
    
    QDomElement root = document.createElementNS( "http://kita.sourceforge.jp/ns/favorites", "favorites" );
    document.appendChild( root );
    
    QDictIterator<Kita::Thread> it( threads() );
    for( ; it.current(); ++it ) {
        Kita::Thread* thread = (*it);
        QDomElement threadElement = document.createElementNS( "http://kita.sourceforge.jp/ns/thread", "thread" );
        root.appendChild( threadElement );
        
        QString datURL = thread->datURL();
        QDomElement datURLElement = document.createElement( "daturl" );
        threadElement.appendChild( datURLElement );
        datURLElement.appendChild( document.createTextNode( datURL ) );
    
        QString threadName = thread->name();
        QDomElement nameElement = document.createElement( "name" );
        threadElement.appendChild( nameElement );
        nameElement.appendChild( document.createTextNode( Kita::unescape( threadName ) ) );
        
        QString resNum = QString::number( thread->resNum() );
        QDomElement resNumElement = document.createElement( "resnum" );
        threadElement.appendChild( resNumElement );
        resNumElement.appendChild( document.createTextNode( resNum ) );
    
        // board    
        QDomElement board = document.createElementNS( "http://kita.sourceforge.jp/ns/board", "board" );
        threadElement.appendChild( board );
        
        QString boardURL = thread->boardURL();
        QDomElement boardURLElement = document.createElement( "url" );
        board.appendChild( boardURLElement );
        boardURLElement.appendChild( document.createTextNode( boardURL ) );
    
        QString boardName = thread->boardName();
        QDomElement boardNameElement = document.createElement( "name" );
        board.appendChild( boardNameElement );
        boardNameElement.appendChild( document.createTextNode( boardName ) );
    }
    return document.toString( 0 );
}
